/*
 * $Header: /home/cvs/jakarta-slide/src/contrib/webdavgui/src/java/org/apache/webdav/ui/filechooser/dir/SPFile.java,v 1.2 2001/07/17 04:00:47 msmith Exp $
 * $Revision: 1.2 $
 * $Date: 2001/07/17 04:00:47 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.webdav.ui.filechooser.dir;

import java.io.*;
import java.util.*;
import javax.swing.JOptionPane;

import org.apache.webdav.lib.*;
import org.apache.webdav.lib.properties.*;
import org.apache.webdav.ui.util.SPUtils;

import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpStatus;

/**
 * Title:        SPFile.java
 * Description:  An adapter for java.io.File
 * Company:      SpeedLegal Holdings Inc.
 * @author       Jojada J. Tirtowidjojo
 * @version 1.0
 */

public class SPFile extends File {
    private SPDirectoryService directoryService;

    public SPFile(String pathname, SPDirectoryService directoryService) {
      super(pathname);
      this.directoryService = directoryService;
    }

    public SPFile(File parent, String child,
	    SPDirectoryService directoryService) {
      super(parent, child);
      this.directoryService = directoryService;
    }

    public SPFile(String parent, String child,
	    SPDirectoryService directoryService) {
      super(parent, child);
      this.directoryService = directoryService;
    }

   /*
    * Determines if the file should be displayed
    * based on the current filter.
    */
    public boolean isAcceptable(String extensionList)  {
	if (isDirectory()) {
	    return true;
	}

	boolean isAcceptable = false;

        String extension = getExtension();

        if (extension != null)  {
          if (extensionList.indexOf(extension) >= 0) {
            //listed
            isAcceptable = true;
          }
        }

        return isAcceptable;
    }

    /*
     * An SPFile which does not exist is a mere abstract path name.
     * Currently the JFileChooser is happy if we assume that this kind
     * of SPFile is NOT a Directory. This is related to the way JFileChooser
     * performs its setSelectedFile() method.
     * setSelectedFile() method calls setCurrentDirectory() which tries to find
     * the current selected file's ancestor directory that exists.
     */

    public boolean isDirectory() {
	if (exists())  {
	    return ( getResourceNode().isWebFolder() );
	}

	return false;
    }

    public boolean isFile() {
	return (! isDirectory() );
    }

    public boolean isTraversable()  {
        return isDirectory();
    }


   /*
    * Returns whether the file is locked.
    * NOTE : The file is locked when it is locked
    *        by other users OR by the current user.
    */
    public boolean isLocked()  {
      return getResourceNode().isLocked();
    }

    public boolean isReadOnly() {
      return getResourceNode().isReadOnly();
    }

    public boolean getReadOnlyCached()  {
      return getResourceNode().getReadOnlyCached();
    }

    public boolean hasReadAccess()  {
      return getResourceNode().hasReadAccess();
    }

    public boolean hasWriteAccess()  {
      return getResourceNode().hasWriteAccess();
    }

    public boolean hasReadWriteAccess()  {
      return getResourceNode().hasReadWriteAccess();
    }

    public boolean exists()  {
        return ( getResourceNode() != null );
    }

    public boolean equals(Object obj) {
	if (obj == null)  {
	    return false;
	}

	SPFile objFile = (SPFile) obj;
        return ( (directoryService == objFile.getDirectoryService())
	     && (super.equals(obj)) );
    }

    public String getResourceUrlPath()  {
        return (getResourceNode().getResourceUrlPath());
    }
    public File getParentFile() {
	if (this.equals(directoryService.getRoot()))  {
	    return this;
	}

	String absPath = getAbsolutePath();
	absPath = absPath.substring(0, absPath.lastIndexOf(File.separator));

	return new SPFile(absPath, directoryService);
    }

   /*
    * Returns the type (i.e : Collection or Non-Collection).
    */
    public int getType()  {
        return getResourceNode().getType();
    }


    public long getContentLength()  {
        GetContentLengthProperty prop = (GetContentLengthProperty)
	    SPUtils.getProperty(getResourceNode(), "getcontentlength");
	return prop.getLength();
    }

    /*
     * Returns null if the resource node is not found in server
     */
    public SPResourceNode getResourceNode()  {
	SPResourceNode theNode=null;

        try {
	    theNode = directoryService.getNodeFromServer(this);

        } catch (Exception e) {
            JOptionPane.showMessageDialog(null, e.getMessage(),
                "File Access Failure", JOptionPane.ERROR_MESSAGE);
	    e.printStackTrace();
        }

	return theNode;

    }

    public SPDirectoryService getDirectoryService()  {
        return directoryService;
    }

  /* Lock this file.
   *
   * @returns a lock token
   */
  public String lock()  {
      return getResourceNode().lock();
  }

  /*
   * Lock this file for 'timeout' seconds
   *
   * @returns a lock token
   */
  public String lock(long timeout)  {
      return getResourceNode().lock(timeout);
  }

  /*
   * UnLocks this file
   *
   * @returns true when succesfull, false otherwise
   */
  public boolean unlock()  {
    return  getResourceNode().unlock();
  }

  public InputStream getWebFileContentAsStream()
	throws Exception {

    return getWebFileContentAsStream(false);
  }

  public InputStream getWebFileContentAsStream(boolean writeLock)
	throws Exception {

    SPResourceNode thisNode = getResourceNode();

    if (thisNode == null)  {
	throw new HttpException("RESOURCE NOT FOUND for File = "
	    +getAbsolutePath());
    }

    if (thisNode.isWebFolder())  {
	return null;
    }
    return ( ((SPWebFile)thisNode).getFileContentAsStream(writeLock)) ;
  }

  public InputStream getWebFileContentAsStream(boolean writeLock,
	long timeout) throws Exception {

    SPResourceNode thisNode = getResourceNode();

    if (thisNode == null)  {
	throw new HttpException("RESOURCE NOT FOUND for File = "
	    +getAbsolutePath());
    }

    if (thisNode.isWebFolder())  {
	return null;
    }

    return ( ((SPWebFile)thisNode).getFileContentAsStream(writeLock, timeout));
  }

  public String getWebFileContentAsString() throws Exception {
    return getWebFileContentAsString(false);
  }

  public String getWebFileContentAsString(boolean writeLock)
	throws Exception {

    SPResourceNode thisNode = getResourceNode();

    if (thisNode == null)  {
	throw new HttpException("RESOURCE NOT FOUND for File = "
	    +getAbsolutePath());
    }

    if (thisNode.isWebFolder())  {
	return null;
    }
    return ( ((SPWebFile)thisNode).getFileContentAsString(writeLock)) ;

  }

  public String getWebFileContentAsString(boolean writeLock,
	long timeout) throws Exception {

    SPResourceNode thisNode = getResourceNode();

    if (thisNode == null)  {
	throw new HttpException("RESOURCE NOT FOUND for File = "
	    +getAbsolutePath());
    }

    if (thisNode.isWebFolder())  {
	return null;
    }
    return ( ((SPWebFile)thisNode).getFileContentAsString(writeLock, timeout));

  }

  public int putWebFileContent(String fileContent) throws Exception  {
    SPResourceNode thisNode = getResourceNode();

    if (thisNode == null)  {
        thisNode = directoryService.createSPResourceNode(this);
    } else if (thisNode.isWebFolder()) {
	return HttpStatus.SC_OK;
    }

    return ( ((SPWebFile)thisNode).putFileContent(fileContent)) ;

  }

  public int putWebFileContent(File aFile) throws Exception  {
    SPResourceNode thisNode = getResourceNode();

    if (thisNode == null)  {
        thisNode = directoryService.createSPResourceNode(this);
    }
    if (thisNode.isWebFolder())  {
	return HttpStatus.SC_OK;
    }

    return ( ((SPWebFile)thisNode).putFileContent(aFile)) ;

  }

  public void setProperties(Map properties) {
    if (isFile())  {
	((SPWebFile)getResourceNode()).setProperties(properties);
    }
  }

  public void refreshContent()  {
    getResourceNode().refreshContent();
  }

  private String getExtension() {
    return SPUtils.getExtension(getName());
  }

} //End of SPFile class










