/*
 * $Header: /home/cvs/jakarta-slide/src/share/org/apache/slide/common/Uri.java,v 1.6 2001/04/29 17:14:25 remm Exp $
 * $Revision: 1.6 $
 * $Date: 2001/04/29 17:14:25 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.slide.common;

import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.io.*;
import org.apache.slide.store.*;

/**
 * This class manages the unique identifier of an object which is 
 * manipulated by Slide.
 * 
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 * @version $Revision: 1.6 $
 */
public final class Uri implements java.io.Serializable {
    
    
    // ----------------------------------------------------------- Constructors
    
    
    /**
     * Constructor.
     * 
     * @param namespace Namespace associated with this Uri
     * @param uri Uri
     */
    public Uri(Namespace namespace, String uri) {
        this(null, namespace, uri);
    }
    
    
    /**
     * Constructor.
     * 
     * @param token Slide token
     * @param namespace Namespace associated with this Uri
     * @param uri Uri
     */
    public Uri(SlideToken token, Namespace namespace, String uri) {
        this.token = token;
        this.namespace = namespace;
        parseUri(uri);
    }


    // ----------------------------------------------------- Instance Variables
    
    
    /**
     * Uri's namespace.
     */
    private transient Namespace namespace;
    
    
    /**
     * uri MUST be unique.
     */
    private transient String uri;
    
    
    /**
     * Scopes to which this Uri belongs.
     */
    private transient ScopeTokenizer scopes;
    
    
    /**
     * FIXME : Is that still used ?
     */
    private transient Scope scope;
    
    
    /**
     * FIXME : Probably useless.
     */
    private transient int pk;
    
    
    /**
     * Associated Store instance.
     */
    private transient Store store;
    
    
    /**
     * Associated SlideToken.
     */
    private transient SlideToken token;
    
    
    // ------------------------------------------------------------- Properties
    
    
    /**
     * Uri mutator.
     *
     * @param uri New uri
     */
    public void setUri(String uri) {
        parseUri(uri);
    }
    
    
    /**
     * Scope accessor.
     * 
     * @return StringTokenizer
     */
    public Scope getScope() {
        return this.scope;
    }
    
    
    /**
     * Returns the scopes tokenized by the ScopeTokenizer associated
     * with this Uri object.
     * 
     * @return Enumeration 
     */
    public Enumeration getScopes() {
        return this.scopes.elements();
    }
    
    
    /**
     * Store accessor.
     * 
     * @return Store
     */
    public Store getStore() {
        return this.store;
    }
    
    
    /**
     * Token accessor.
     * 
     * @return The SlideToken, or null if no token has been set 
     * (which is valid)
     */
    public SlideToken getToken() {
        return token;
    }
    
    
    /**
     * Token mutator.
     * 
     * @param token New Slide token
     */
    public void setToken(SlideToken token) {
        this.token = token;
    }
    
    
    // --------------------------------------------------------- Public Methods
    
    
    /**
     * Return the namespace to which this Uri belongs.
     * 
     * @return Namespace
     */
    public Namespace getNamespace() {
        return namespace;
    }
    
    
    /**
     * Get the parent uri.
     * 
     * @return Uri
     */
    public Uri getParentUri() {
        Uri result = scopes.getParentUri();
        if (result != null)
            result.setToken(token);
        return result;
    }
    
    
    /**
     * Invalidate the current Services and PK. Used if there are changes in 
     * the namespace at runtime.
     */
    public void invalidateServices() {
        store = null;
        pk = 0;
        parseUri(this.uri);
    }
    
    
    /**
     * Reconnect the Uri services.
     */
    public void reconnectServices() {
        try {
            store.connectIfNeeded();
        } catch (ServiceConnectionFailedException e) {
            parseUri(this.uri);
        } catch (ServiceAccessException e) {
            parseUri(this.uri);
        }
    }
    
    
    /**
     * Get the relative path to the matched scope.
     */
    public String getRelative() {
        return (uri.substring(scope.toString().length()));
    }
    
    
    // --------------------------------------------------------- Object Methods
    
    
    /**
     * String representation of the uri object.
     * 
     * @return String
     */
    public String toString() {
        return uri;
    }
    
    
    /**
     * Hash code.
     * 
     * @return int hash code
     */
    public int hashCode() {
	return pk;
    }
    
    
    /**
     * Tests equivalence of two Uris.
     * 
     * @param obj Object to test
     * @return boolean
     */
    public boolean equals(Object obj) {
        if ((obj != null) && (obj instanceof Uri)) {
            return (uri.equals(obj.toString()));
        } else {
            return false;
        }
    }
    
    
    /**
     * Tests if the given uri represents a parent object.
     * 
     * @param uri Uri to test
     * @return boolean
     */
    public boolean isParent(Uri uri) {
        return this.uri.startsWith(uri.toString());
    }
    
    
    // -------------------------------------------------------- Private Methods
    
    
    /**
     * This function is called by the constructor and when uri is changed.
     * 
     * @param uri Uri to parse
     */
    private void parseUri(String uri) {
	// We first try to tokenize the uri string.
	
	scopes = new ScopeTokenizer(namespace, uri);
	
        this.uri = scopes.getUri();
        
	// Find the qualifiying stuff from the registry.
	// Then we contentStore the scope of the found Data Source 
        // within scope.
	store = null;
	while ((store == null) && (scopes.hasMoreElements())) {
	    Scope courScope = scopes.nextScope();
	    try {
                if (store == null) {
		    store = namespace.retrieveStore(courScope);
                    if (store != null) {
                        scope = courScope;
                        // The PK in the current descriptorsStore is the 
                        // portion of the URI relative
                        // to the Data Source.
                        this.pk = uri.substring(scope.toString().length())
                            .hashCode();
                    }
		}
	    } catch (ServiceConnectionFailedException e) {
	        // Problem ...
                // FIXME : Throw a RuntimeException ??
	    } catch (ServiceAccessException e) {
	        // Problem ...
                // FIXME : Throw a RuntimeException ??
	    }
        }
        
        // If descriptorsStore or contentStore is still null, then no valid 
        // scope is defined in the namespace ...
        if (store == null) {
            throw new ServiceMissingOnRootNodeException();
        }
        
    }
    
    
    // ---------------------------------------------------------- Serialization
    
    
    /**
     * Read serialized object.
     */
    private void readObject(java.io.ObjectInputStream in)
    throws IOException, ClassNotFoundException {
        String namespaceName = (String) in.readObject();
        namespace = Domain.getNamespace(namespaceName);
        parseUri((String) in.readObject());
    }
    
    
    /**
     * Write serialized object.
     */
    private void writeObject(java.io.ObjectOutputStream out)
        throws IOException {
        out.writeObject(namespace.getName());
        out.writeObject(uri);
    }
    
}
