/*
 * $Header: /home/cvs/jakarta-slide/src/share/org/apache/slide/util/logger/Logger.java,v 1.3 2001/03/19 16:40:28 juergen Exp $
 * $Revision: 1.3 $
 * $Date: 2001/03/19 16:40:28 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package org.apache.slide.util.logger;

/**
 * The <code>Logger</code> block interface.
 * <p />
 * This interface must be implemented by those blocks willing to log data
 * coming from other blocks.
 * <p />
 * The six levels used for logging are (in order of gravity): EMERGENCY,
 * CRITICAL, ERROR, WARNING, INFO and DEBUG.
 *
 * @version 1.0 (CVS $Revision: 1.3 $ $Date: 2001/03/19 16:40:28 $)
 * @author <a href="mailto:scoobie@pop.systemy.it">Federico Barbieri</a>
 * @author <a href="mailto:pier@apache.org">Pierpaolo Fumagalli</a>
 * @author <a href="mailto:stefano@apache.org">Stefano Mazzocchi</a>
 * @author <a href="http://java.apache.org/">Java Apache Project</a>
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 */
public interface Logger {
    
    
    // -------------------------------------------------------------- Constants
    
    
    /**
     * The EMERGENCY logging level (anything logged under this level means
     * that a <b>critical permanent error</b> was detected).
     * <p />
     * When a <code>Block</code> logs something with this level means that
     * any other task will be impossible without administrator's intervention.
     * <p />
     * Example:
     * <ul>
     * <li><i>
     * AJP handler cannot find the secret key file. Cannot process servlet
     * requests.
     * </i></li>
     * <li><i>
     * Cannot access to mail server database URL jdbc:mysql://localhost/users
     * (SQLException caught in driver initialization)
     * </i></li>
     * </ul>
     */
    public static final int EMERGENCY=0;
    
    
    /**
     * The CRITICAL logging level (anything logged under this level means that
     * a <b>critical temporary error</b> was detected).
     * <p />
     * When a <code>Block</code> logs something with this level means that
     * tasks similar to the one generating the error will be impossible
     * without administrator's intervention, but other tasks (maybe using
     * different resources) will be possible.
     * <p />
     * Example:
     * <ul>
     * <li><i>
     * Cannot serve servlet in zone &quot;myServletZone&quot; (repository not
     * found)
     * </i></li>
     * <li><i>
     * Cannot alias address &quot;listserv@apache.org&quot; to ListServlet
     * (mail servlet not found)
     * </i></li>
     * </ul>
     */
    public static final int CRITICAL=1;
    
    
    /**
     * The ERROR logging level (anything logged under this level means that a
     * <b>non-critical error</b> was detected).
     * <p />
     * When a <code>Block</code> logs something with this level means that
     * the current tasks failed to execute, but the stability of the block is
     * not compromised.
     * <p />
     * Example:
     * <ul>
     * <li><i>
     * Cannot serve servlet &quot;SnoopServlet&quot; to client 192.168.1.1 due
     * to security restriction
     * </i></li>
     * <li><i>
     * User &quot;ianosh&quot; denied access via POP3 protocol due to invalid
     * password
     * </i></li>
     * </ul>
     */
    public static final int ERROR=2;

    /**
     * The WARNING logging level (anything logged under this level means that
     * a <b>generic error</b> was detected).
     * <p />
     * When a <code>Block</code> logs something with this level means that
     * the current tasks failed to execute due to inconsistencies in the
     * request handled by the block.
     * <p />
     * Example:
     * <ul>
     * <li><i>
     * Cannot find servlet &quot;SnoopServlet&quot; requested by 192.168.1.1
     * via AJP protocol
     * </i></li>
     * <li><i>
     * Cannot deliver mail from &quot;root@localhost&quot; to
     * &quot;ianosh@iname.com&quot; via protocol SMTP (user not found)
     * </i></li>
     * </ul>
     */
    public static final int WARNING=4;
    
    
    /**
     * The INFO logging level (anything logged under this level means that
     * an <b>informative message</b> must be logged).
     * <p />
     * When a <code>Block</code> logs something with this level means that
     * something that may be interesting happened.
     * <p />
     * Example:
     * <ul>
     * <li><i>
     * Successfully handled request from 192.168.1.1 via AJP protocol for
     * &quot;SnoopServlet&quot; servlet
     * </i></li>
     * <li><i>
     * Mail from &quot;root@localhost&quot; delivered successfully to
     * &quot;ianosh@iname.com&quot; via protocol SMTP
     * </i></li>
     * </ul>
     */
    public static final int INFO=6;
    
    
    /**
     * The DEBUG logging level (anything logged under this level means that
     * an <b>debug message</b> must be logged).
     * <p />
     * When a <code>Block</code> logs something with this level means that
     * something that may be interesting while developing, tracking bugs or
     * misconfigurations.
     * <p />
     * <p />
     * Example:
     * <ul>
     * <li><i>
     * Requested servlet &quot;snoop&quot; was aliased to
     * &quot;org.apache.servlets.SnoopServlet&quot;
     * </i></li>
     * <li><i>
     * MX record for domain &quot;dom.tld&quot; resolved to host
     * &quot;mail.dom.tld&quot; (192.168.1.1) with preference level 10
     * </i></li>
     * </ul>
     */
    public static final int DEBUG=7;
    
    
    /**
     * Specify the default string for logging without a channel.
     *
     * @param data The object to log.
     * @param channel The channel name used for logging.
     * @param level The level used for logging.
     */
    public static String DEFAULT_CHANNEL = "org.apache.slide.util.logger";
    
    
    // ------------------------------------------------------ Interface Methods
    
    
    /**
     * Log an object thru the specified channel and with the specified level.
     *
     * @param data The object to log.
     * @param channel The channel name used for logging.
     * @param level The level used for logging.
     */
    public void log(Object data, String channel, int level);
    
    
    /**
     * Log an object with the specified level.
     *
     * @param data The object to log.
     * @param level The level used for logging.
     */
    public void log(Object data, int level);
    
    
    /**
     * Log an object.
     *
     * @param data The object to log.
     */
    public void log(Object data);
    
        
    
    /**
     * Set the logger level for the default channel
     *
     * @param data The object to log.
     */

    public void setLoggerLevel(int level);
    
    
            
    /**
     * Set the logger level for the specified channel
     *
     * @param data The object to log.
     */

    public void setLoggerLevel(String channel, int level);
    
        
    
    /**
     * Get the logger level for the default channel
     *
     * @param data The object to log.
     */

    public int getLoggerLevel();
    
    
            
    /**
     * Get the logger level for the specified channel
     *
     * @param data The object to log.
     */

    public int getLoggerLevel(String channel);
            
    
        
    
    /**
     * Check if the channel with the specified level is enabled for logging.
     *
     * @param channel The channel specification
     * @param level   The level specification
     */

    public boolean isEnabled(String channel, int level);
            
        
    
    /**
     * Check if the default channel with the specified level is enabled for logging.
     *
     * @param level   The level specification
     */

    public boolean isEnabled(int level);
            
}
